@@ -16,6 +16,7 @@ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLI
 #include <vsg/core/Object.h>
 #include <vsg/core/type_name.h>
 #include <vsg/maths/mat4.h>
+#include <vsg/vk/Slots.h>
 
 #include <set>
 #include <vector>
@@ -62,6 +63,9 @@ namespace vsg
     class CommandGraph;
     class RecordedCommandBuffers;
     class Instrumentation;
+    class InstanceNode;
+    class InstanceDraw;
+    class InstanceDrawIndexed;
 
     VSG_type_name(vsg::RecordTraversal);
 
@@ -69,7 +73,7 @@ namespace vsg
     class VSG_DECLSPEC RecordTraversal : public Object
     {
     public:
-        explicit RecordTraversal(uint32_t in_maxSlot = 2, const std::set<Bin*>& in_bins = {});
+        explicit RecordTraversal(const Slots& in_maxSlots = {}, const std::set<Bin*>& in_bins = {});
 
         RecordTraversal(const RecordTraversal&) = delete;
         RecordTraversal& operator=(const RecordTraversal& rhs) = delete;
@@ -86,13 +90,16 @@ namespace vsg
         Mask traversalMask = MASK_ALL;
         Mask overrideMask = MASK_OFF;
 
+        /// Light::intensity minimum value for a light to be passed to GPU.
+        float intensityMinimum = 0.001f;
+
         ref_ptr<Instrumentation> instrumentation;
 
         /// Container for CommandBuffers that have been recorded in current frame
         ref_ptr<RecordedCommandBuffers> recordedCommandBuffers;
 
         /// get the current State object used to track state and projection/modelview matrices for the current subgraph being traversed
-        State* getState() { return _state; }
+        State* getState() { return state; }
 
         /// get the current CommandBuffer for the current subgraph being traversed
         CommandBuffer* getCommandBuffer();
@@ -101,10 +108,10 @@ namespace vsg
         uint32_t deviceID() const;
 
         void setFrameStamp(FrameStamp* fs);
-        FrameStamp* getFrameStamp() { return _frameStamp; }
+        FrameStamp* getFrameStamp() { return frameStamp; }
 
         void setDatabasePager(DatabasePager* dp);
-        DatabasePager* getDatabasePager() { return _databasePager; }
+        DatabasePager* getDatabasePager() { return databasePager; }
 
         void apply(const Object& object);
 
@@ -141,6 +148,11 @@ namespace vsg
         // Animation nodes
         void apply(const Joint& joint);
 
+        // instance nodes
+        void apply(const InstanceNode& instanceNode);
+        void apply(const InstanceDraw& instanceDraw);
+        void apply(const InstanceDrawIndexed& instanceDrawIndexed);
+
         // Vulkan nodes
         void apply(const StateGroup& object);
 
@@ -161,19 +173,19 @@ namespace vsg
         // list of pairs of modelview matrix & region of interest
         std::vector<std::pair<dmat4, const RegionOfInterest*>> regionsOfInterest;
 
-    protected:
-        virtual ~RecordTraversal();
-
-        ref_ptr<FrameStamp> _frameStamp;
-        ref_ptr<State> _state;
+        ref_ptr<FrameStamp> frameStamp;
+        ref_ptr<State> state;
 
         // used to handle loading of PagedLOD external children.
-        ref_ptr<DatabasePager> _databasePager;
-        ref_ptr<CulledPagedLODs> _culledPagedLODs;
+        ref_ptr<DatabasePager> databasePager;
+        ref_ptr<CulledPagedLODs> culledPagedLODs;
 
-        int32_t _minimumBinNumber = 0;
-        std::vector<ref_ptr<Bin>> _bins;
-        ref_ptr<ViewDependentState> _viewDependentState;
+        int32_t minimumBinNumber = 0;
+        std::vector<ref_ptr<Bin>> bins;
+        ref_ptr<ViewDependentState> viewDependentState;
+
+    protected:
+        virtual ~RecordTraversal();
     };
 
 } // namespace vsg
