@@ -23,6 +23,7 @@ namespace vsg
     // forward declare core Objects
     class Objects;
     class External;
+    class MipmapLayout;
 
     // forward declare node classes
     class Node;
@@ -34,8 +35,9 @@ namespace vsg
     class StateGroup;
     class CullGroup;
     class CullNode;
-    class MatrixTransform;
     class Transform;
+    class MatrixTransform;
+    class CoordinateFrame;
     class Geometry;
     class VertexDraw;
     class VertexIndexDraw;
@@ -50,6 +52,9 @@ namespace vsg
     class SpotLight;
     class InstrumentationNode;
     class RegionOfInterest;
+    class InstanceNode;
+    class InstanceDraw;
+    class InstanceDrawIndexed;
 
     // forward declare text classes
     class Text;
@@ -62,6 +67,7 @@ namespace vsg
     class AnimationGroup;
     class AnimationSampler;
     class TransformSampler;
+    class CameraSampler;
     class MorphSampler;
     class JointSampler;
     class Joint;
@@ -156,6 +162,7 @@ namespace vsg
     class Viewer;
     class ViewMatrix;
     class LookAt;
+    class LookDirection;
     class RelativeViewMatrix;
     class TrackingViewMatrix;
     class ProjectionMatrix;
@@ -187,6 +194,7 @@ namespace vsg
         virtual void apply(Objects&);
         virtual void apply(External&);
         virtual void apply(Data&);
+        virtual void apply(MipmapLayout&);
 
         // Values
         virtual void apply(stringValue&);
@@ -220,10 +228,15 @@ namespace vsg
         virtual void apply(uivec2Value&);
         virtual void apply(uivec3Value&);
         virtual void apply(uivec4Value&);
+        virtual void apply(mat2Value&);
+        virtual void apply(dmat2Value&);
+        virtual void apply(mat3Value&);
+        virtual void apply(dmat3Value&);
         virtual void apply(mat4Value&);
         virtual void apply(dmat4Value&);
 
         // Arrays
+        virtual void apply(stringArray&);
         virtual void apply(byteArray&);
         virtual void apply(ubyteArray&);
         virtual void apply(shortArray&);
@@ -262,8 +275,11 @@ namespace vsg
         virtual void apply(block128Array&);
 
         // Array2Ds
+        virtual void apply(byteArray2D&);
         virtual void apply(ubyteArray2D&);
+        virtual void apply(shortArray2D&);
         virtual void apply(ushortArray2D&);
+        virtual void apply(intArray2D&);
         virtual void apply(uintArray2D&);
         virtual void apply(floatArray2D&);
         virtual void apply(doubleArray2D&);
@@ -295,8 +311,11 @@ namespace vsg
         virtual void apply(block128Array2D&);
 
         // Array3Ds
+        virtual void apply(byteArray3D&);
         virtual void apply(ubyteArray3D&);
+        virtual void apply(shortArray3D&);
         virtual void apply(ushortArray3D&);
+        virtual void apply(intArray3D&);
         virtual void apply(uintArray3D&);
         virtual void apply(floatArray3D&);
         virtual void apply(doubleArray3D&);
@@ -323,8 +342,9 @@ namespace vsg
         virtual void apply(StateGroup&);
         virtual void apply(CullGroup&);
         virtual void apply(CullNode&);
-        virtual void apply(MatrixTransform&);
         virtual void apply(Transform&);
+        virtual void apply(MatrixTransform&);
+        virtual void apply(CoordinateFrame&);
         virtual void apply(Geometry&);
         virtual void apply(VertexDraw&);
         virtual void apply(VertexIndexDraw&);
@@ -339,6 +359,9 @@ namespace vsg
         virtual void apply(SpotLight&);
         virtual void apply(InstrumentationNode&);
         virtual void apply(RegionOfInterest&);
+        virtual void apply(InstanceNode&);
+        virtual void apply(InstanceDraw&);
+        virtual void apply(InstanceDrawIndexed&);
 
         // text
         virtual void apply(Text&);
@@ -353,6 +376,7 @@ namespace vsg
         virtual void apply(JointSampler&);
         virtual void apply(MorphSampler&);
         virtual void apply(TransformSampler&);
+        virtual void apply(CameraSampler&);
         virtual void apply(Joint&);
 
         // Vulkan nodes
@@ -444,6 +468,7 @@ namespace vsg
         virtual void apply(Viewer&);
         virtual void apply(ViewMatrix&);
         virtual void apply(LookAt&);
+        virtual void apply(LookDirection&);
         virtual void apply(RelativeViewMatrix&);
         virtual void apply(TrackingViewMatrix&);
         virtual void apply(ProjectionMatrix&);
