@@ -35,7 +35,7 @@ Viewer::Viewer() :
     _start_point(clock::now()),
     _frameStamp(FrameStamp::create(_start_point, 0, 0.0))
 {
-    CPU_INSTRUMENTATION_L1_NC(instrumentation, "Viewer costructor", COLOR_VIEWER);
+    CPU_INSTRUMENTATION_L1_NC(instrumentation, "Viewer constructor", COLOR_VIEWER);
 }
 
 Viewer::~Viewer()
@@ -361,7 +361,7 @@ void Viewer::compile(ref_ptr<ResourceHints> hints)
 
         for (const auto& commandGraph : task->commandGraphs)
         {
-            commandGraph->maxSlot = resourceRequirements.maxSlot;
+            commandGraph->maxSlots = resourceRequirements.maxSlots;
             if (resourceRequirements.containsPagedLOD) task_containsPagedLOD = true;
         }
 
@@ -524,13 +524,10 @@ void Viewer::assignRecordAndSubmitTaskAndPresentation(CommandGraphs in_commandGr
 
             Windows activeWindows(findWindows.windows.begin(), findWindows.windows.end());
 
-            auto renderFinishedSemaphore = vsg::Semaphore::create(device);
-
             // set up Submission with CommandBuffer and signals
             auto recordAndSubmitTask = vsg::RecordAndSubmitTask::create(device, numBuffers);
             recordAndSubmitTask->commandGraphs = commandGraphs;
             recordAndSubmitTask->databasePager = databasePager;
-            recordAndSubmitTask->signalSemaphores.emplace_back(renderFinishedSemaphore);
             recordAndSubmitTask->windows = activeWindows;
             recordAndSubmitTask->queue = mainQueue;
             recordAndSubmitTasks.emplace_back(recordAndSubmitTask);
@@ -541,7 +538,6 @@ void Viewer::assignRecordAndSubmitTaskAndPresentation(CommandGraphs in_commandGr
             if (instrumentation) recordAndSubmitTask->assignInstrumentation(instrumentation);
 
             auto presentation = vsg::Presentation::create();
-            presentation->waitSemaphores.emplace_back(renderFinishedSemaphore);
             presentation->windows = activeWindows;
             presentation->queue = device->getQueue(deviceQueueFamily.presentFamily);
             presentations.emplace_back(presentation);
@@ -750,7 +746,7 @@ void Viewer::setupThreading()
                 if (i == 0)
                     threads.emplace_back(run_primary, sharedData, task->commandGraphs[i], make_string("Viewer primary thread"));
                 else
-                    threads.emplace_back(run_secondary, sharedData, task->commandGraphs[i], make_string("Viewer seconary thread ", i));
+                    threads.emplace_back(run_secondary, sharedData, task->commandGraphs[i], make_string("Viewer secondary thread ", i));
             }
 
             if (task->transferTask)
