@@ -37,6 +37,11 @@ void ConstVisitor::apply(const Data& value)
     apply(static_cast<const Object&>(value));
 }
 
+void ConstVisitor::apply(const MipmapLayout& value)
+{
+    apply(static_cast<const Object&>(value));
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 //
 // Values
@@ -165,6 +170,22 @@ void ConstVisitor::apply(const uivec4Value& value)
 {
     apply(static_cast<const Data&>(value));
 }
+void ConstVisitor::apply(const mat2Value& value)
+{
+    apply(static_cast<const Data&>(value));
+}
+void ConstVisitor::apply(const dmat2Value& value)
+{
+    apply(static_cast<const Data&>(value));
+}
+void ConstVisitor::apply(const mat3Value& value)
+{
+    apply(static_cast<const Data&>(value));
+}
+void ConstVisitor::apply(const dmat3Value& value)
+{
+    apply(static_cast<const Data&>(value));
+}
 void ConstVisitor::apply(const mat4Value& value)
 {
     apply(static_cast<const Data&>(value));
@@ -178,6 +199,10 @@ void ConstVisitor::apply(const dmat4Value& value)
 //
 // Arrays
 //
+void ConstVisitor::apply(const stringArray& value)
+{
+    apply(static_cast<const Data&>(value));
+}
 void ConstVisitor::apply(const byteArray& value)
 {
     apply(static_cast<const Data&>(value));
@@ -657,6 +682,18 @@ void ConstVisitor::apply(const RegionOfInterest& value)
 {
     apply(static_cast<const Node&>(value));
 }
+void ConstVisitor::apply(const InstanceNode& value)
+{
+    apply(static_cast<const Compilable&>(value));
+}
+void ConstVisitor::apply(const InstanceDraw& value)
+{
+    apply(static_cast<const Command&>(value));
+}
+void ConstVisitor::apply(const InstanceDrawIndexed& value)
+{
+    apply(static_cast<const Command&>(value));
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -818,7 +855,7 @@ void ConstVisitor::apply(const RayTracingPipeline& value)
 }
 void ConstVisitor::apply(const GraphicsPipelineState& value)
 {
-    apply(static_cast<const Object&>(value));
+    apply(static_cast<const StateCommand&>(value));
 }
 void ConstVisitor::apply(const ShaderStage& value)
 {
