@@ -37,6 +37,11 @@ void Visitor::apply(Data& value)
     apply(static_cast<Object&>(value));
 }
 
+void Visitor::apply(MipmapLayout& value)
+{
+    apply(static_cast<Object&>(value));
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 //
 // Values
@@ -165,6 +170,22 @@ void Visitor::apply(uivec4Value& value)
 {
     apply(static_cast<Data&>(value));
 }
+void Visitor::apply(mat2Value& value)
+{
+    apply(static_cast<Data&>(value));
+}
+void Visitor::apply(dmat2Value& value)
+{
+    apply(static_cast<Data&>(value));
+}
+void Visitor::apply(mat3Value& value)
+{
+    apply(static_cast<Data&>(value));
+}
+void Visitor::apply(dmat3Value& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(mat4Value& value)
 {
     apply(static_cast<Data&>(value));
@@ -178,6 +199,10 @@ void Visitor::apply(dmat4Value& value)
 //
 // Arrays
 //
+void Visitor::apply(stringArray& value)
+{
+    apply(static_cast<Data&>(value));
+}
 void Visitor::apply(byteArray& value)
 {
     apply(static_cast<Data&>(value));
@@ -657,6 +682,18 @@ void Visitor::apply(RegionOfInterest& value)
 {
     apply(static_cast<Node&>(value));
 }
+void Visitor::apply(InstanceNode& value)
+{
+    apply(static_cast<Compilable&>(value));
+}
+void Visitor::apply(InstanceDraw& value)
+{
+    apply(static_cast<Command&>(value));
+}
+void Visitor::apply(InstanceDrawIndexed& value)
+{
+    apply(static_cast<Command&>(value));
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -818,7 +855,7 @@ void Visitor::apply(RayTracingPipeline& value)
 }
 void Visitor::apply(GraphicsPipelineState& value)
 {
-    apply(static_cast<Object&>(value));
+    apply(static_cast<StateCommand&>(value));
 }
 void Visitor::apply(ShaderStage& value)
 {
