@@ -29,10 +29,12 @@ ObjectFactory::ObjectFactory()
     add<vsg::Object>();
     add<vsg::Objects>();
     add<vsg::External>();
+    add<vsg::MipmapLayout>();
 
     // values
     add<vsg::stringValue>();
     add<vsg::wstringValue>();
+    add<vsg::pathValue>();
     add<vsg::boolValue>();
     add<vsg::intValue>();
     add<vsg::uintValue>();
@@ -58,12 +60,13 @@ ObjectFactory::ObjectFactory()
     add<vsg::materialValue>();
     add<vsg::PhongMaterialValue>();
     add<vsg::PbrMaterialValue>();
-    add<vsg::sphereValue>();
-    add<vsg::boxValue>();
+    add<vsg::TexCoordIndicesValue>();
     add<vsg::quatValue>();
-    add<vsg::dsphereValue>();
-    add<vsg::dboxValue>();
     add<vsg::dquatValue>();
+    add<vsg::sphereValue>();
+    add<vsg::dsphereValue>();
+    add<vsg::boxValue>();
+    add<vsg::dboxValue>();
 
     // arrays
     add<vsg::byteArray>();
@@ -100,12 +103,15 @@ ObjectFactory::ObjectFactory()
     add<vsg::uivec4Array>();
     add<vsg::mat4Array>();
     add<vsg::dmat4Array>();
+    add<vsg::quatArray>();
+    add<vsg::dquatValue>();
     add<vsg::block64Array>();
     add<vsg::block128Array>();
     add<vsg::materialArray>();
     add<vsg::PhongMaterialArray>();
     add<vsg::PbrMaterialArray>();
     add<vsg::DrawIndirectCommandArray>();
+    add<vsg::DrawIndexedIndirectCommandArray>();
 
     // array2Ds
     add<vsg::byteArray2D>();
@@ -187,6 +193,9 @@ ObjectFactory::ObjectFactory()
     add<vsg::TileDatabase>();
     add<vsg::TileDatabaseSettings>();
     add<vsg::InstrumentationNode>();
+    add<vsg::InstanceNode>();
+    add<vsg::InstanceDraw>();
+    add<vsg::InstanceDrawIndexed>();
 
     // lighting
     add<vsg::Light>();
@@ -320,10 +329,11 @@ ObjectFactory::ObjectFactory()
     // utils
     add<vsg::ShaderSet>();
     add<vsg::ViewDependentStateBinding>();
-    add<vsg::PositionAndDisplacementMapArrayState>();
-    add<vsg::DisplacementMapArrayState>();
-    add<vsg::PositionArrayState>();
     add<vsg::BillboardArrayState>();
+    add<vsg::TranslationArrayState>();
+    add<vsg::TranslationRotationScaleArrayState>();
+    add<vsg::TranslationAndDisplacementMapArrayState>();
+    add<vsg::DisplacementMapArrayState>();
     add<vsg::SharedObjects>();
     add<vsg::ProfileLog>();
 
@@ -344,5 +354,6 @@ vsg::ref_ptr<vsg::Object> ObjectFactory::create(const std::string& className)
     }
 
     warn("ObjectFactory::create(", className, ") failed to find means to create object.");
+
     return vsg::ref_ptr<vsg::Object>();
 }
