@@ -12,6 +12,7 @@ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLI
 
 #include <vsg/io/VSG.h>
 #include <vsg/io/glsl.h>
+#include <vsg/io/json.h>
 #include <vsg/io/read.h>
 #include <vsg/io/spirv.h>
 #include <vsg/io/tile.h>
@@ -50,6 +51,11 @@ ref_ptr<Object> vsg::read(const Path& filename, ref_ptr<const Options> options)
             spirv rw;
             return rw.read(filename, options);
         }
+        else if (ext == ".json")
+        {
+            json rw;
+            return rw.read(filename, options);
+        }
         else if (glsl::extensionSupported(ext))
         {
             glsl rw;
