@@ -26,14 +26,8 @@ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLI
 #include <vsg/nodes/TileDatabase.h>
 #include <vsg/nodes/VertexIndexDraw.h>
 #include <vsg/state/BindDescriptorSet.h>
-#include <vsg/state/ColorBlendState.h>
-#include <vsg/state/DepthStencilState.h>
 #include <vsg/state/DescriptorImage.h>
 #include <vsg/state/GraphicsPipeline.h>
-#include <vsg/state/InputAssemblyState.h>
-#include <vsg/state/MultisampleState.h>
-#include <vsg/state/RasterizationState.h>
-#include <vsg/state/VertexInputState.h>
 #include <vsg/state/material.h>
 #include <vsg/ui/UIEvent.h>
 #include <vsg/utils/ComputeBounds.h>
@@ -169,6 +163,10 @@ vsg::ref_ptr<vsg::Object> tile::read_root(vsg::ref_ptr<const vsg::Options> optio
                 {
                     imageData = settings->imageLayerCallback(imageData);
                 }
+                else if (!imageData)
+                {
+                    vsg::warn("tile::read_root() unable read image data, imagePath = ", imagePath);
+                }
             }
 
             if (settings->detailLayer)
@@ -179,6 +177,10 @@ vsg::ref_ptr<vsg::Object> tile::read_root(vsg::ref_ptr<const vsg::Options> optio
                 {
                     detailData = settings->detailLayerCallback(detailData);
                 }
+                else if (!detailData)
+                {
+                    vsg::warn("tile::read_root() unable read detail data, detailPath = ", detailPath);
+                }
             }
 
             if (settings->elevationLayer)
@@ -189,6 +191,10 @@ vsg::ref_ptr<vsg::Object> tile::read_root(vsg::ref_ptr<const vsg::Options> optio
                 {
                     elevationData = settings->elevationLayerCallback(elevationData);
                 }
+                else if (!elevationData)
+                {
+                    vsg::warn("tile::read_root() unable read elevation data, terrainPath = ", terrainPath);
+                }
             }
 
             auto tile_extents = computeTileExtents(x, y, lod);
@@ -470,7 +476,7 @@ void tile::init(vsg::ref_ptr<const vsg::Options> options)
         _graphicsPipelineConfig->enableTexture("displacementMap");
         _graphicsPipelineConfig->enableDescriptor("displacementMapScale");
 
-        auto elevationData = vsg::floatValue::create(0.0f);
+        auto elevationData = vsg::floatArray2D::create(1, 1, 0.0f);
         elevationData->properties.format = VK_FORMAT_R32_SFLOAT;
         _elevationFallback = vsg::DescriptorImage::create(_sampler, elevationData, 7, 0, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER);
     }
@@ -505,6 +511,20 @@ void tile::init(vsg::ref_ptr<const vsg::Options> options)
         _material = vsg::DescriptorBuffer::create(mat, 10, 0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER);
     }
 
+    if (auto& texCoordIndicesBinding = _shaderSet->getDescriptorBinding("texCoordIndices"))
+    {
+        // Assumed texCoordIndicesBinding.set is equal to _materialSetIndex used to assign the tile texture and texCoordIndices
+
+        ref_ptr<Data> indices = texCoordIndicesBinding.data;
+        if (!indices) indices = vsg::TexCoordIndicesValue::create();
+        _texCoordIndices = vsg::DescriptorBuffer::create(indices, texCoordIndicesBinding.binding, 0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER);
+    }
+    else
+    {
+        auto indices = vsg::TexCoordIndicesValue::create();
+        _texCoordIndices = vsg::DescriptorBuffer::create(indices, 11, 0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER);
+    }
+
     _graphicsPipelineConfig->init();
 }
 
@@ -558,6 +578,7 @@ ref_ptr<BindDescriptorSet> tile::createBindDescriptorSet(ref_ptr<Data> imageData
     }
 
     descriptors.push_back(_material);
+    descriptors.push_back(_texCoordIndices);
 
     return vsg::BindDescriptorSet::create(VK_PIPELINE_BIND_POINT_GRAPHICS, _graphicsPipelineConfig->layout, _materialSetIndex, descriptors);
 }
@@ -760,7 +781,7 @@ vsg::ref_ptr<vsg::Node> tile::createECEFTile(const vsg::dbox& tile_extents, ref_
                 (*itr++) = tile_i + 1;
             }
 
-            // colum[0]
+            // column[0]
             uint32_t tile_left_column = 0;
             uint32_t skirt_left_column = vi;
             for (uint32_t r = 0; r < numRows; ++r, ++vi)
