@@ -78,9 +78,9 @@ struct AssignGraphicsPipelineStates : public vsg::Visitor
         vertexInputState = VertexInputState::create(ias);
         config->pipelineStates.push_back(vertexInputState);
     }
-    void apply(vsg::ViewportState& ias) override
+    void apply(vsg::ViewportState& vs) override
     {
-        viewportState = ViewportState::create(ias);
+        viewportState = ViewportState::create(vs);
         config->pipelineStates.push_back(viewportState);
     }
 };
@@ -151,9 +151,15 @@ bool DescriptorConfigurator::assignTexture(const std::string& name, ref_ptr<Data
                 textureData->properties.format = vsg::sRGB_to_uNorm(textureData->properties.format);
         }
 
+        auto descriptorImage = DescriptorImage::create(sampler, textureData ? textureData : textureBinding.data, textureBinding.binding, dstArrayElement, textureBinding.descriptorType);
+
+        for (auto& imageInfo : descriptorImage->imageInfoList)
+        {
+            imageInfo->computeNumMipMapLevels();
+        }
+
         // create texture image and associated DescriptorSets and binding
-        return assignDescriptor(textureBinding.set, textureBinding.binding, textureBinding.descriptorType, textureBinding.descriptorCount, textureBinding.stageFlags,
-                                DescriptorImage::create(sampler, textureData ? textureData : textureBinding.data, textureBinding.binding, dstArrayElement, textureBinding.descriptorType));
+        return assignDescriptor(textureBinding.set, textureBinding.binding, textureBinding.descriptorType, textureBinding.descriptorCount, textureBinding.stageFlags, descriptorImage);
     }
     return false;
 }
@@ -167,7 +173,7 @@ bool DescriptorConfigurator::assignTexture(const std::string& name, const ImageI
         // set up bindings
         if (!textureBinding.define.empty()) defines.insert(textureBinding.define);
 
-        for (auto& imageInfo : imageInfoList)
+        for (const auto& imageInfo : imageInfoList)
         {
             if (imageInfo->imageView && imageInfo->imageView->image)
             {
@@ -182,9 +188,15 @@ bool DescriptorConfigurator::assignTexture(const std::string& name, const ImageI
             }
         }
 
+        auto descriptorImage = DescriptorImage::create(imageInfoList, textureBinding.binding, dstArrayElement, textureBinding.descriptorType);
+
+        for (auto& imageInfo : descriptorImage->imageInfoList)
+        {
+            imageInfo->computeNumMipMapLevels();
+        }
+
         // create texture image and associated DescriptorSets and binding
-        return assignDescriptor(textureBinding.set, textureBinding.binding, textureBinding.descriptorType, textureBinding.descriptorCount, textureBinding.stageFlags,
-                                DescriptorImage::create(imageInfoList, textureBinding.binding, dstArrayElement, textureBinding.descriptorType));
+        return assignDescriptor(textureBinding.set, textureBinding.binding, textureBinding.descriptorType, textureBinding.descriptorCount, textureBinding.stageFlags, descriptorImage);
     }
     return false;
 }
@@ -663,6 +675,20 @@ bool GraphicsPipelineConfigurator::copyTo(StateCommands& stateCommands, ref_ptr<
                 {
                     if (sharedObjects)
                     {
+                        for (auto& descriptor : ds->descriptors)
+                        {
+                            if (auto descriptor_image = descriptor.cast<vsg::DescriptorImage>())
+                            {
+                                for (auto& image_info : descriptor_image->imageInfoList)
+                                {
+                                    if (image_info->imageView && image_info->imageView->image)
+                                    {
+                                        sharedObjects->share(image_info->imageView->image);
+                                    }
+                                }
+                            }
+                            sharedObjects->share(descriptor);
+                        }
                         sharedObjects->share(ds->setLayout);
                         sharedObjects->share(ds);
                         sharedObjects->share(bindDescriptorSet);
