@@ -194,14 +194,23 @@ void Intersector::apply(const BufferInfo& bufferInfo)
     if (bufferInfo.data) bufferInfo.data->accept(*this);
 }
 
+void Intersector::apply(const ubyteArray& array)
+{
+    ubyte_indices = &array;
+    ushort_indices = nullptr;
+    uint_indices = nullptr;
+}
+
 void Intersector::apply(const ushortArray& array)
 {
+    ubyte_indices = nullptr;
     ushort_indices = &array;
     uint_indices = nullptr;
 }
 
 void Intersector::apply(const uintArray& array)
 {
+    ubyte_indices = nullptr;
     ushort_indices = nullptr;
     uint_indices = &array;
 }
@@ -209,9 +218,15 @@ void Intersector::apply(const uintArray& array)
 void Intersector::apply(const TextTechnique& technique)
 {
     if (auto cpuTechnique = technique.cast<CpuLayoutTechnique>())
-        cpuTechnique->scenegraph->accept(*this);
+    {
+        if (cpuTechnique->scenegraph)
+            cpuTechnique->scenegraph->accept(*this);
+    }
     if (auto gpuTechnique = technique.cast<GpuLayoutTechnique>())
-        gpuTechnique->scenegraph->accept(*this);
+    {
+        if (gpuTechnique->scenegraph)
+            gpuTechnique->scenegraph->accept(*this);
+    }
 }
 
 void Intersector::apply(const Draw& draw)
