@@ -12,7 +12,9 @@ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLI
 
 #include <vsg/core/Exception.h>
 #include <vsg/io/Logger.h>
+#include <vsg/utils/Profiler.h>
 #include <vsg/vk/CommandBuffer.h>
+#include <vsg/vk/State.h>
 
 using namespace vsg;
 
@@ -40,12 +42,31 @@ CommandBuffer::~CommandBuffer()
     }
 }
 
-void CommandBuffer::reset()
+void CommandBuffer::reset(VkCommandBufferResetFlags flags)
 {
     _currentPipelineLayout = VK_NULL_HANDLE;
     _currentPushConstantStageFlags = 0;
 
-    _commandPool->reset();
+    if ((_commandPool->flags & VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT) != 0)
+    {
+        vkResetCommandBuffer(_commandBuffer, flags);
+    }
+}
+
+void CommandBuffer::setCurrentPipelineLayout(const PipelineLayout* pipelineLayout)
+{
+    VkPipelineLayout newLayout = pipelineLayout->vk(deviceID);
+    if (_currentPipelineLayout != newLayout)
+    {
+        // have to assume that all DescriptorSets will need to be rebound.
+        state->dirtyStateStacks();
+
+        _currentPipelineLayout = newLayout;
+        if (pipelineLayout->pushConstantRanges.empty())
+            _currentPushConstantStageFlags = 0;
+        else
+            _currentPushConstantStageFlags = pipelineLayout->pushConstantRanges.front().stageFlags;
+    }
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
