@@ -23,7 +23,6 @@ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLI
 #include <vsg/nodes/VertexDraw.h>
 #include <vsg/nodes/VertexIndexDraw.h>
 #include <vsg/state/DescriptorImage.h>
-#include <vsg/state/MultisampleState.h>
 #include <vsg/state/ViewDependentState.h>
 #include <vsg/utils/ShaderSet.h>
 #include <vsg/vk/Context.h>
@@ -64,7 +63,7 @@ DescriptorPoolSizes ResourceRequirements::computeDescriptorPoolSizes() const
 
 void ResourceRequirements::apply(const ResourceHints& resourceHints)
 {
-    if (resourceHints.maxSlot > maxSlot) maxSlot = resourceHints.maxSlot;
+    maxSlots.merge(resourceHints.maxSlots);
 
     if (!resourceHints.descriptorPoolSizes.empty() || resourceHints.numDescriptorSets > 0)
     {
@@ -85,6 +84,7 @@ void ResourceRequirements::apply(const ResourceHints& resourceHints)
     shadowMapSize = std::max(shadowMapSize, resourceHints.shadowMapSize);
 
     dataTransferHint = resourceHints.dataTransferHint;
+    viewportStateHint = resourceHints.viewportStateHint;
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -95,7 +95,7 @@ ref_ptr<ResourceHints> CollectResourceRequirements::createResourceHints(uint32_t
 {
     auto resourceHints = vsg::ResourceHints::create();
 
-    resourceHints->maxSlot = requirements.maxSlot;
+    resourceHints->maxSlots = requirements.maxSlots;
     resourceHints->numDescriptorSets = static_cast<uint32_t>(requirements.computeNumDescriptorSets() * tileMultiplier);
     resourceHints->descriptorPoolSizes = requirements.computeDescriptorPoolSizes();
 
@@ -154,8 +154,7 @@ void CollectResourceRequirements::apply(const PagedLOD& plod)
 
 void CollectResourceRequirements::apply(const StateCommand& stateCommand)
 {
-    if (stateCommand.slot > requirements.maxSlot) requirements.maxSlot = stateCommand.slot;
-
+    if (stateCommand.slot > requirements.maxSlots.state) requirements.maxSlots.state = stateCommand.slot;
     stateCommand.traverse(*this);
 }
 
@@ -212,8 +211,18 @@ void CollectResourceRequirements::apply(const Light& light)
     requirements.viewDetailsStack.top().lights.insert(&light);
 }
 
+void CollectResourceRequirements::apply(const RenderGraph& rg)
+{
+    if (rg.viewportState) requirements.maxSlots.view = std::max(requirements.maxSlots.view, rg.viewportState->slot);
+    rg.traverse(*this);
+}
+
 void CollectResourceRequirements::apply(const View& view)
 {
+    if (view.camera && view.camera->viewportState)
+    {
+        requirements.maxSlots.view = std::max(requirements.maxSlots.view, view.camera->viewportState->slot);
+    }
 
     if (auto itr = requirements.views.find(&view); itr != requirements.views.end())
     {
@@ -237,8 +246,6 @@ void CollectResourceRequirements::apply(const View& view)
 
     if (view.viewDependentState)
     {
-        if (requirements.maxSlot < 2) requirements.maxSlot = 2;
-
         view.viewDependentState->init(requirements);
 
         view.viewDependentState->accept(*this);
